import json


class Model:
    class Meta:
        strict_validation = False

    def __init__(self, json_dict: dict = None, json_str: str = None):
        if json_dict:
            self.raw = json_dict
        elif json_str:
            json.loads(json_str)
        else:
            raise ValueError('Either json_dict or json_str is required.')



from typing import List, Union

import jsonpath_rw
from typeguard import check_type


class Attribute:
    def __init__(self, path: str):
        self._path = path
        self._expr = jsonpath_rw.parse(self._path)

    def __get__(self, obj, obj_type=None):
        return self._format(
            result=self._expr.find(obj.raw),
            path=self._path,
            strict_validation=obj.Meta.strict_validation)

    @staticmethod
    def _format(result: list, path: str, strict_validation: bool):
        raise NotImplemented


class StringAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> str:
        formatted = result[0].value

        if strict_validation:
            check_type(path, formatted, str, memo=None)

        return formatted


class StringListAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> List[str]:
        formatted = list(map(lambda x: x.value, result))

        if strict_validation:
            check_type(path, formatted, List[str], memo=None)

        return list(map(lambda x: x.value, result))


class NumberAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> Union[int, float]:
        formatted = result[0].value

        if strict_validation:
            check_type(path, formatted, Union[int, float], memo=None)

        return formatted


class NumberListAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> List[Union[int, float]]:
        formatted = list(map(lambda x: x.value, result))

        if strict_validation:
            check_type(path, formatted, List[Union[int, float]], memo=None)

        return formatted


class BooleanAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> bool:
        formatted = result[0].value

        if strict_validation:
            check_type(path, formatted, bool, memo=None)

        return formatted


class BooleanListAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> List[bool]:
        formatted = list(map(lambda x: x.value, result))

        if strict_validation:
            check_type(path, formatted, List[bool], memo=None)

        return formatted


class ListAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> list:
        formatted = list(map(lambda x: x.value, result))

        if strict_validation:
            check_type(path, formatted, list, memo=None)

        return formatted


class ObjectAttribute(Attribute):
    @staticmethod
    def _format(result: list, path: str, strict_validation: bool) -> dict:
        formatted = dict([(k, v) for k, v in result[0].value.items()])

        if strict_validation:
            check_type(path, formatted, dict, memo=None)

        return formatted



import unittest

from pyojm.models import Model
from pyojm.attributes import *


class TestJsonModel(unittest.TestCase):
    def test_positive_01(self):

        data = {
            'string_a': 'foo',
            'string_list_a': ['foo', 'bar'],
            'number_a': 1,
            'number_list_a': [1, 2],
            'boolean_a': True,
            'boolean_list_a': [True, False],
            'list_a': ['foo', 1],
            'object_a': {
                'foo': 'bar'
            }
        }

        class TestModel(Model):
            class Meta:
                strict_validation = True
            string_a = StringAttribute('string_a')
            string_list_a = StringListAttribute('string_list_a.[*]')
            number_a = NumberAttribute('number_a')
            number_list_a = NumberListAttribute('number_list_a.[*]')
            boolean_a = BooleanAttribute('boolean_a')
            boolean_list_a = BooleanListAttribute('boolean_list_a.[*]')
            list_a = ListAttribute('list_a.[*]')
            object_a = ObjectAttribute('object_a')

        t = TestModel(data)

        self.assertEqual('foo', t.string_a)
        self.assertEqual(['foo', 'bar'], t.string_list_a)
        self.assertEqual(1, t.number_a)
        self.assertEqual([1, 2], t.number_list_a)
        self.assertEqual(True, t.boolean_a)
        self.assertEqual([True, False], t.boolean_list_a)
        self.assertEqual(['foo', 1], t.list_a)
        self.assertEqual({
            'foo': 'bar'
        }, t.object_a)

    def test_negative_01(self):
        data = {
            'string_a': 1
        }

        class TestModel(Model):
            class Meta:
                strict_validation = True
            string_a = StringAttribute('string_a')

        t = TestModel(data)
        with self.assertRaises(TypeError):
            print(t.string_a)



import sys
from setuptools import setup, find_packages
sys.path.append('./pyojm')
sys.path.append('./tests')

install_requires = [
    'jsonpath_rw',
    'typeguard'
]

setup(
    name='pyojm',
    version="0.1.0",
    author='lune*',
    author_email='lune@wuvu.net',
    description='A Pythonic Interface to Json.',
    license='MIT',
    keywords='json wrapper model',
    install_requires=install_requires,
    url='https://github.com/lune_sta/json-model',
    packages=find_packages(exclude='tests'),
    test_suite='tests'
)

